LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

-- This entity is a register with multiple inputs.
-- When address is 00, the register maintains its value.
-- When address is 01, the register increments itself.
-- When address is 10, the register loads the value on Data1.
-- When address is 11, the register loads the value on Data2.
ENTITY pcounter IS
	PORT(
		Jmp, Acc: IN	STD_LOGIC;							-- reset, clock
		Inputs: IN STD_LOGIC_VECTOR(3 downto 0);			-- action selector
		Q : OUT	STD_LOGIC_VECTOR(3 downto 0));				-- output
END pcounter;

ARCHITECTURE behavioral OF pcounter IS
	SIGNAL Qreg: UNSIGNED (3 downto 0);	-- internal register representation
BEGIN
	action:	PROCESS(Inputs, Jmp, Acc) BEGIN
		if (Jmp = '1' and Acc = '1') then
			Qreg <= UNSIGNED(Inputs) + 1;
		else Qreg <= UNSIGNED(Inputs);
		end if;										-- value logic, this keeps the synthesis simple
	END PROCESS action;

	Q <= STD_LOGIC_VECTOR(Qreg);	-- write the register value to the output
									-- you could put an output enable switch here as well.
END behavioral;



