LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY wholething IS
	PORT(Clk, Rst, p_button: 	IN	STD_LOGIC;		-- reset, clock
		Q : 	OUT	STD_LOGIC_VECTOR(7 downto 0));
END wholething;

ARCHITECTURE structural OF wholething IS
	component not8                                   -- the not gates 
	port( a: in std_logic_vector (7 downto 0);
			b: out std_logic_vector (7 downto 0) );
	end component;

	component pause_button
	port( clk, p_button: in std_logic;
			clk_out: out std_logic);
	end component;
	
	component lightreg 
	port(Clk, Rst: 	IN	STD_LOGIC;							-- reset, clock
		Sel: 		IN STD_LOGIC_VECTOR(1 downto 0);		-- action selector
		Inst: 		IN	STD_LOGIC_VECTOR(4 downto 0);		-- alternative load inputs
		Acc: 		IN STD_LOGIC_VECTOR(7 DOWNTO 0);   		
		Q : 		OUT	STD_LOGIC_VECTOR(7 downto 0));		-- output
	end component;
	component accumulator_reg 
	port (	clk, rst : in std_logic;
			instruction_reg: in std_logic_vector (4 downto 0);
			light_reg: in std_logic_vector (7 downto 0);
			selector: in std_logic_vector (2 downto 0);
			q: out std_logic_vector (7 downto 0));
	end component;
	component instruction_reg
	port (
			clk, rst: in std_logic;
			data: in std_logic_vector (14 downto 0);
			inst_out: out std_logic_vector (14 downto 0));
	end component;
	component prog
		port(pc: in std_logic_vector (3 downto 0);
		data: out std_logic_vector (14 downto 0));
	end component;
	component pcounter
	PORT(
		Jmp, Acc: IN	STD_LOGIC;							-- reset, clock
		Inputs: IN STD_LOGIC_VECTOR(3 downto 0);			-- action selector
		Q : OUT	STD_LOGIC_VECTOR(3 downto 0));				-- output
	end component;		
	component clockdivider
		port(reset, clk: in std_logic;  -- reset and clock input signals
             clockbits: out std_logic_vector(7 downto 0));  -- output clock signals
	end component;
	
	signal pause_button_out: std_logic;
	signal light_reg_out : std_logic_vector(7 downto 0);
	signal accumulator_out : std_logic_vector(7 downto 0);
	signal inst_out			: std_logic_vector(14 downto 0);
	signal program_out		: std_logic_vector(14 downto 0);
	signal pc_out			: std_logic_vector(3 downto 0);
	signal clock_out		: std_logic_vector(7 downto 0);
BEGIN
	pause_button_0: pause_button port map( clk, p_button, pause_button_out ); 
	clockdivider_0: clockdivider port map (rst, pause_button_out, clock_out);
	pcounter_0: pcounter port map (inst_out(14), accumulator_out(0), inst_out(3 downto 0), pc_out);
	prog_0: prog port map (pc_out, program_out);
	instruction_reg_0: instruction_reg port map (clock_out(7), rst, program_out, inst_out);
	accumulator_0: accumulator_reg port map(clock_out(7), rst, inst_out(8 downto 4), light_reg_out, 
											inst_out(13 downto 11), accumulator_out);
	lightreg_0: lightreg port map(clock_out(7), Rst, inst_out(10 downto 9), inst_out(8 downto 4), 
									accumulator_out, light_reg_out);
	not8_0 : not8 port map(light_reg_out, Q);
END structural;




